#!/bin/ksh -p
#-----------------------------------------------------------------------
# (C) COPYRIGHT International Business Machines Corp. 2001-2009
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# VERSION: db2_v97fp4 s110330
# NAME: mountV97_monitor.ksh /mount_point [ verbose ]
#
# FUNCTION: Monitor method for filesystem in TSA cluster
#
# Return 1 if mount point is mounted and accessible (ie. online)
# Return 2 if mount point is not mounted and underlying VG varyedoff
# Return 3 if mount point is mounted but IO errors
# Return 4 if no /etc/fstab,/etc/filesystems entry
#
# Return 0 (Unknown) VG varied on but fs not mounted
#-----------------------------------------------------------------------
PATH=/bin:/usr/bin:/sbin:$PATH
export CT_MANAGEMENT_SCOPE=2

myname=$(basename $0)
dirname=$(dirname $0)

MP=${1}
VERBOSE=${2:-noverbose}

UNAME=$(uname)

#logger -i -p debug -t $myname "Entering monitor for $MP"

if [[ "$VERBOSE" == "verbose" ]]; then
#  exec   2> /dev/null
   set -x
else
   #   Close stdout   and   stderr
   exec   2> /dev/null
   exec   1> /dev/null
   set +x
fi

if [[ -z $MP ]]; then
   rc=3
   logger -i -p err -t $0 "Error: Must specify mount point"
   logger -i -p err -t $myname "Returning $rc for mount $MP"
   echo $rc
   return $rc
fi

function isMounted
{
   let mounted=0
   if [[ "${UNAME}" == "AIX" ]]; then
      mounted=$(mount | awk '{print $2}' | grep -c "^${MP}\$")
   elif [[ "${UNAME}" == "SunOS" ]]; then
      mounted=$(mount | awk '{print $1}' | grep -c "^${MP}\$")
   else
      mounted=$(mount | awk '{print $3}' | grep -c "^${MP}\$")
   fi
   if [[ -z $mounted ]]; then
      mounted=0
   fi
#logger -i -p debug -t $myname "isMounted returns $mounted"
   return $mounted
}

if [[ "${UNAME}" == "AIX" ]]; then
   #AIX
   isAutoMountSet=$(cat /etc/filesystems | grep -p ${MP?}: | awk '{print $1 " " $2 " " $3}' | egrep "^mount = " | awk '{print $3}' |  egrep -i -c "(automatic|true)")

   if [[ $isAutoMountSet -eq 1 ]]; then
      logger -i -p err -t $myname "$MP is set to true. Please change this."
      isMounted
      if [[ $mounted -gt 0 ]]; then
         exit 4
      else
         exit 3
      fi
   fi

   blockDevice=$(cat /etc/filesystems | grep -p ${MP?}: | awk '{print $1 " " $2 " " $3}' | egrep "^dev = " | awk '{print $3}')
   #logger -i -p debug -t $myname "blockDevice equals $blockDevice"

   if [[ -z $blockDevice ]]; then
      # This is not a valid mount point, it may be an RLV
      blockDevice=${MP?}
      unset MP
      rlv=true
   fi
   lvname=${blockDevice#/dev/}
   vgname=$(lslv $lvname -L 2> /dev/null | grep "VOLUME GROUP" | awk '{print $6}')
   if [[ ! -z "$vgname" ]] ; then
      majNum=$(ls -l /dev/$vgname | tr "," " " | awk '{print $5}')
      lspvLine=$(lspv -L | grep " $vgname ")
      diskID=$(echo $lspvLine | awk '{print $2}')
      hdiskID=$(echo $lspvLine | awk '{print $1}')
      lvname=${blockDevice#/dev/}
      #logger -i -p debug -t $myname "majNum,lspvLine,diskID,hdiskID,lvname equals $majNum,$lspvLine,$diskID,$hdiskID,$lvname"
   else
      logger -i -p err -t $myname "Mount point $MP not defined at this host. Check /etc/filesystems"
      logger -i -p err -t $myname "majNum,lspvLine,diskID,hdiskID,lvname equals $majNum,$lspvLine,$diskID,$hdiskID,$lvname"
      isMounted
      if [[ $mounted -gt 0 ]]; then
         # This is bad ... not in /etc/filesystem yet mounted
         exit 4
      else
         exit 3
      fi
   fi
   concurrent=$(lsvg $vgname | grep "^Concurrent:" | awk '{print $2}')

elif [[ $UNAME == "SunOS" ]]; then
   #SunOS

   isAutoMountSet=$(cat /etc/vfstab | awk '{print $3 " " $6}' | grep "^${MP} " | egrep -c "yes")
   if [[ $isAutoMountSet -eq 1 ]]; then
      logger -i -p err -t $myname "$MP is set to yes. Please change this."
      isMounted   
      if [[ $mounted -gt 0 ]]; then
         exit 4
      else
         exit 3
      fi
   fi
   poolName=$(cat /etc/vfstab | awk '{print $1 " " $3}' | grep " ${MP}$" |  awk '{print $1}' )
   #logger -i -p debug -t $myname "poolName is $poolName"
   isMounted   
   if [[ -z "$poolName" ]]; then
      logger -i -p err -t $myname "Mount point $MP not defined at this host. Check /etc/vfstab."
      if [[ $mounted -gt 0 ]]; then
         exit 4
      else
         exit 3
      fi
   fi
else
   # Linux

   isAutoMountSet=$(cat /etc/fstab | awk '{print $2 " " $4}' | grep "^${MP} " | egrep -c "noauto")
   if [[ $isAutoMountSet -eq 0 ]]; then
      logger -i -p err -t $myname "$MP is not set to noauto. Please change this."
      isMounted
      if [[ $mounted -gt 0 ]]; then
         exit 4
      else
         exit 3
      fi
   fi
   blockDevice=$(cat /etc/fstab | awk '{print $1 " " $2}' | grep " ${MP}$" | awk '{print $1}')
   #logger -i -p debug -t $myname "blockDevice is $blockDevice"
   isMounted
   if [[ -z "$blockDevice" ]]; then
      logger -i -p err -t $myname "Mount point $MP not defined at this host.  Check /etc/fstab."
      if [[ $mounted -gt 0 ]]; then
         exit 4
      else
         exit 3
      fi
   fi
fi

if [[ ! -z "$blockDevice" ]]; then
   cd $MP
   rc=$?
   cd /
   if [[ $rc -ne 0 ]]; then
      logger -i -p err -t $myname "Mount point stub $MP not defined at this host.  You may need to do a mkdir $MP at all nodes."
      exit 3
   fi
fi


###################################################################
# Start main...
let mounted=0
let rc=2
if [[ $rlv == "true" ]]; then
   # This will be case when MP passed in is actually a RLV
   vgactive=$(echo $lspvLine | awk '{print $4}')
   if [[ "X$vgactive" == "Xactive" ]];then
      rc=1
      echo $rc
      return $rc
   else
      rc=2
      echo $rc
      return $rc
   fi
fi

# What is status of this mount point?
isMounted
if [ ${mounted} -gt 0 ]; then
   touch ${MP}/.mp_monitor.$$
   if [ -f ${MP}/.mp_monitor.$$ ]; then
      # logger -i -p debug -t $myname "No error performing I/O to file ${MP}/.mp_monitor.$$"
      rc=1
      rm -f ${MP}/.mp_monitor.$$
   else
      logger -i -p err -t $myname "Error performing I/O to file ${MP}/.mp_monitor.$$"
      rm -f ${MP}/.mp_monitor.$$
      $dirname/mountV97_stop.ksh $MP force
      rc=3
   fi
fi

# If AIX, what is status of the underlying VG if fs not mounted
if [[ "${UNAME}" == "AIX" && $concurrent != "Enhanced-Capable" ]]; then
   if [[ $rc -eq 2 ]]; then
      vgactive=$(echo $lspvLine | awk '{print $4}')
      #logger -i -p debug -t $myname "vgactive is $vgactive"
      if [[ "X$vgactive" == "Xactive" ]];then
         logger -i -p debug -t $myname "VG $vgname is online at this node"
         msip=$(ps -ef | grep -v grep | egrep -c "mountV97_start.ksh|mountV97_stop.ksh")
         if [[ $msip -eq 0 ]]; then
            mount $MP
         fi
         rc=0
         echo $rc
      fi
   fi
fi

# If a psuedo-offline file is present, return offline
if [ -f ${MP}/.mp_pseudo ]; then
   logger -i -p notice -t $myname "$MP/.mp_pseudo exists at this node.  Returning 2"
   rc=2
fi

logger -i -p debug -t $myname "Returning $rc for mount $MP"
echo $rc
return $rc
