#!/bin/ksh
#-----------------------------------------------------------------------
# (C) COPYRIGHT International Business Machines Corp. 2001-2009
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# VERSION: db2_v97fp4 s110330
# NAME: mountV97_start.ksh mount_point [ verbose ]
#
# FUNCTION: Start method for filesystem in TSA cluster
# Return 0 if mount point is mounted and accessible
# Return non-zero otherwise
#
#-----------------------------------------------------------------------
PATH=/bin:/usr/bin:/sbin:$PATH
export CT_MANAGEMENT_SCOPE=2

myname=$(basename $0)
dirname=$(dirname $0)
MP=${1}
VERBOSE=${2:-noverbose}

UNAME=$(uname)

logger -i -p notice -t $myname "Entered ($MP)"

if [[ "$VERBOSE" == "verbose" ]]; then
   typeset -ft $(typeset +f)
   set -x
else
   # Close stdout and stderr
   exec 2> /dev/null
   exec 1> /dev/null
   set +x
fi

# Verify input...
if [[ -z $MP ]]; then
   logger -i -p err -t $myname "Error: Must specify mount point"
   rc=2
   logger -i -p err -t $myname "Returning $rc"
   return $rc
fi

function isMounted
{
   let mounted=0
   if [[ "${UNAME}" == "AIX" ]]; then
      mounted=$(mount | awk '{print $2}' | grep -c "^${MP}\$")
   elif [[ "${UNAME}" == "SunOS" ]]; then
      mounted=$(mount | awk '{print $1}' | grep -c "^${MP}\$")
   else
      mounted=$(mount | awk '{print $3}' | grep -c "^${MP}\$")
   fi
   if [[ -z $mounted ]]; then
      mounted=0
   fi
   logger -i -p debug -t $myname "isMounted for $MP returns $mounted"
   return $mounted
}

if [[ $UNAME == "AIX" ]]; then
   #AIX
   until [[ -r /etc/filesystems ]]; do 
      logger -i -p err -t $myname "Error: /etc/filesystems not readable"
      sleep 1
   done
   blockDevice=$(cat /etc/filesystems | grep -p ${MP?}: | awk '{print $1 " " $2 " " $3}' | egrep "^dev = " | awk '{print $3}')
   logger -i -p debug -t $myname "blockDevice is $blockDevice"

   if [[ -z $blockDevice ]]; then
      # This is not a valid mount point, it may be an RLV
      blockDevice=${MP?}
      unset MP
      rlv=true
   fi
   lvname=${blockDevice#/dev/}
   vgname=$(lslv $lvname 2> /dev/null | grep "VOLUME GROUP" | awk '{print $6}')
   logger -i -p debug -t $myname "lvname is $lvname, vgname is $vgname"
   if [[ -z "$vgname" ]]; then
      logger -i -p err -t $myname "Cannot find input mount point or RLV.  Exiting."
      exit 2
   else
      majNum=$(ls -l /dev/$vgname | tr "," " " | awk '{print $5}')
      diskID=$(lspv | grep " $vgname " | awk '{print $2}')
      hdiskID=$(lspv | grep " $vgname " | awk '{print $1}')
      lvname=${blockDevice#/dev/}
      logger -i -p debug -t $myname "$majNum,$diskID,$hdiskID,$lvname"
      vgActive=$(lsvg -p ${vgname?} | grep -c "active")
      if [[ $vgActive == 0 ]]; then
         varyonvg ${vgname?}
         rc=$?
         logger -i -p debug -t $myname "varyonvg ${vgname} returns $rc"
         if [[ $rc -ne 0 ]]; then
            varyonvg -b ${vgname?}
            rc=$?
            logger -i -p debug -t $myname "varyonvg -b ${vgname} returns $rc"
         fi
      else
         logger -i -p debug -t $myname "vg ${vgname} is already active"
      fi
   fi

elif [[ $UNAME == "SunOS" ]]; then
   #SunOS
   poolName=$(cat /etc/vfstab | awk '{print $1 " " $3}' | grep " ${MP}$" |  awk '{print $1}' )
   logger -i -p debug -t $myname "poolName is $poolName"
   if [[ -z "$poolName" ]]; then
      logger -i -p err -t $myname "Mount point $MP not defined at this host"
      exit 2
   fi
   zpool_already_imported=$(zpool list | awk '{print $1}' | grep -c "^${poolName?}$")
   logger -i -p debug -t $myname "zpool ${poolName}: $zpool_already_imported"
   if [[ ${zpool_already_imported?} -eq 0 ]]; then 
      zpool import ${poolName}
      rc=$?
      logger -i -p debug -t $myname "zpool import ${poolName} $rc"

      if [[ $rc -ne 0 ]] then
         logger -i -p err -t $myname "Could not import zpool ${poolName}, zpool import ${poolName} returns $rc"
         exit 2
      fi
   fi
else
   blockDevice=$(cat /etc/fstab | awk '{print $1 " " $2}' | grep " ${MP}$" | awk '{print $1}')
   logger -i -p debug -t $myname "blockDevice ${blockDevice}"
   if [[ -z $blockDevice ]]; then
      if [[ -r /etc/fstab ]]; then
         logger -i -p err -t $myname "Mount point $MP not defined at this host"
         exit 2
      fi
   fi
fi

###################################################################
# Start main...
rc=0

# If already mounted correctly...
if [[ $rlv == "true" ]]; then
   # RLV
   logger -i -p debug -t $myname "rlv is $rlv"
   return 0
fi

isMounted
if [ ${mounted} -gt 0 ]; then
   logger -i -p info -t $myname "$MP is already mounted"
   rm -f ${MP}/.mp_pseudo
   rm -f ${MP}/.mp_monitor*
   touch ${MP}/.mp_monitor 
   rc=$?
   logger -i -p debug -t $myname "touch $MP/.mp_monitor returns $rc"
else
      if [[ $UNAME == "AIX" ]]; then
         vgActive=$(lsvg -p ${vgname?} | grep -c "active")
         if [[ $vgActive == 0 ]]; then
            varyonvg ${vgname?}
            rc=$?
            logger -i -p debug -t $myname "varyonvg ${vgname} returns $rc"
            if [[ $rc -ne 0 ]]; then
               varyonvg -b ${vgname?}
               rc=$?
               logger -i -p debug -t $myname "varyonvg -b ${vgname} returns $rc"
            fi
         else
            logger -i -p debug -t $myname "vg ${vgname} is already active"
         fi
      fi
      mount ${MP}
      rc=$?
      logger -i -p debug -t $myname "mount $MP returns $rc"

      if [ $rc -eq 0 ]; then
         rm -f ${MP}/.mp_pseudo
         rm -f ${MP}/.mp_monitor*
         touch ${MP}/.mp_monitor
         rc=$?
         logger -i -p debug -t $myname "touch $MP/.mp_monitor returns $rc"
         if [ $rc -ne 0 ]; then
            logger -i -p err -t $myname "$MP not writeable: $rc"
         fi
      else
         logger -i -p err -t $myname "$MP not mountable: $rc"

         # Safely fsck the disk
         logger -i -p notice -t $myname "Will fsck ${blockDevice?}"
         if [[ $UNAME == "AIX" ]]; then
            fsck -f -p ${blockDevice?}
            rc=$?
         else
            fsck -p ${blockDevice?}
            rc=$?
         fi
         logger -i -p notice -t $myname "fsck ${blockDevice?} returns $rc"

         if [[ $UNAME == "AIX" ]]; then
            varyonvg -b ${vgname?}
            rc=$?
         fi
         mount ${MP}
         rc=$?
         while [[ $rc -ne 0 ]];
         do 
            logger -i -p err -t $myname "$MP not mountable: mount attempt fails: $rc"
            sleep 1
            mount ${MP}
            rc=$?
         done

         if [[ $rc -eq 0 ]]; then
            logger -i -p notice -t $myname "$MP mountable: mount succeeds: $rc"
            rm -f ${MP}/.mp_pseudo
            rm -f ${MP}/.mp_monitor*
            touch ${MP}/.mp_monitor
            rc=$?
            if [ $rc -ne 0 ]; then
               logger -i -p err -t $myname "$MP not writeable: $rc"
            fi
         fi
      fi
fi

logger -i -p notice -t $myname "Returning $rc for $MP"
return $rc
