#!/bin/ksh -p
#-----------------------------------------------------------------------
# (C) COPYRIGHT International Business Machines Corp. 2001-2009
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# VERSION: db2_v97fp4 s110330
# NAME: mountV97_stop.ksh /mount_point [force] [verbose]
#
# FUNCTION: Stop method for filesystem in TSA cluster
#
# Return 0 if filesystem is unmounted cleanly (and in AIX, 
#  underlying VG is varied off)
#
# Do not return until filesystem is no longer mounted at this node
# Force option will skip the soft unmount
#
#-----------------------------------------------------------------------
PATH=/bin:/usr/bin:/sbin:$PATH
export CT_MANAGEMENT_SCOPE=2

myname=$(basename $0)
dirname=$(dirname $0)
MP=${1}

UNAME=$(uname)

logger -i -p notice -t $myname "Entered $MP"

let force=0
if [[ "$2" == "force" ]]; then
   force=1
fi

if [[ "$2" == "verbose" || "$3" == "verbose" ]]; then
   typeset -ft $(typeset +f)
   set -x
else
   # Close stdout and stderr
   exec 2> /dev/null
   exec 1> /dev/null
   set +x
fi

if [[ -z $MP ]]; then
   logger -i -p err -t $myname "Error: Must specify mount point"
   return 2
fi

function isMounted
{
   let mounted=0
   if [[ "${UNAME}" == "AIX" ]]; then
      mounted=$(mount | awk '{print $2}' | grep -c "^${MP}\$")
   elif [[ "${UNAME}" == "SunOS" ]]; then
    mounted=$(mount | awk '{print $1}' | grep -c "^${MP}\$")
   else
      mounted=$(mount | awk '{print $3}' | grep -c "^${MP}\$")
   fi

   if [[ -z $mounted ]]; then
      mounted=0
   fi
   logger -i -p debug -t $myname "isMounted for $MP returns $mounted"

   return $mounted
}

if [[ $UNAME == "AIX" ]]; then
   #AIX
   until [[ -r /etc/filesystems ]]; do
      logger -i -p err -t $myname "Error: /etc/filesystems not readable"
      sleep 1
   done
   blockDevice=$(cat /etc/filesystems | grep -p ${MP?}: | awk '{print $1 " " $2 " " $3}' | egrep "^dev = " | awk '{print $3}')
   logger -i -p debug -t $myname "blockDevice is $blockDevice"

   if [[ -z $blockDevice ]]; then
      # This is not a valid mount point, it may be an RLV
      blockDevice=${MP?}
      unset MP
      rlv=true
   fi
   lvname=${blockDevice#/dev/}
   vgname=$(lslv $lvname 2> /dev/null | grep "VOLUME GROUP" | awk '{print $6}')
   logger -i -p debug -t $myname "lvname is $lvname, vgname is $vgname"

   if [[ ! -z "$vgname" ]] ; then
      majNum=$(ls -l /dev/$vgname | tr "," " " | awk '{print $5}')
      diskID=$(lspv | grep " $vgname " | awk '{print $2}')
      hdiskID=$(lspv | grep " $vgname " | awk '{print $1}')
      lvname=${blockDevice#/dev/}
      logger -i -p debug -t $myname "$majNum,$diskID,$hdiskID,$lvname"
   else
      logger -i -p err -t $myname "Mount point $MP not defined at this host"
      isMounted
      exit $mounted
   fi
   concurrent=$(lsvg $vgname | grep "^Concurrent:" | awk '{print $2}')
elif [[ $UNAME == "SunOS" ]]; then
     #SunOS
     poolName=$(cat /etc/vfstab | awk '{print $1 " " $3}' | grep " ${MP}$" | awk '{print $1}' )

     if [[ -z "$poolName" ]]; then
        logger -i -p err -t $myname "Mount point $MP not defined at this host"
        exit 2
     fi
else
   blockDevice=$(cat /etc/fstab | awk '{print $1 " " $2}' | grep " ${MP}$" | awk '{print $1}')
   if [[ -z $blockDevice ]]; then
      if [[ -r /etc/fstab ]]; then
         logger -i -p err -t $myname "Mount point $MP not defined at this host"
         isMounted
         exit $mounted
      fi
   fi
fi

###################################################################
# Start main...
rc=0

if [[ $rlv == "true" && $concurrent != "Enhanced-Capable" ]]; then
   # RLV
   logger -i -p notice -t $myname "$vgname is RLV"
   varyoffvg ${vgname?}
   rc=$?
   logger -i -p notice -t $myname "varyoffvg $vgname returns $rc"
   return $rc
fi

isMounted
if [[ $mounted == 0 ]]; then
   logger -i -p info -t $myname "Filesystem ${MP} isn't mounted on this node" 
else
   # If rg exists for this MP, touch file and exit
   DATA_SVC_RG_NAME=$(echo "db2mnt"${MP?} | tr "/" "_")
   rc1=$(lsrsrc -s "Name == '${DATA_SVC_RG_NAME?}'" IBM.Test Name | grep -c ${DATA_SVC_RG_NAME?})
   if [[ $rc1 -eq 1 ]]; then
      logger -i -p notice -t $myname "Pseudo Offline file exists"
      touch ${MP}/.mp_pseudo
   else
      # Clean up dir and umount it
      rm -f ${MP}/.mp_monitor
      umount $MP
      doKillAlso=$?
      isMounted

      ((softUnmounts=0))
      # Only attempt soft unmount if 'force' option is not used
      while [ $force != 1 -a $mounted != 0 -a $softUnmounts != 10 ];
      do
         if [[ "${UNAME}" == "AIX" ]]; then
               exportfs -u ${MP}
               list_of_pids_to_be_killed=$(fuser -c $MP)
         elif [[ "${UNAME}" == "SunOS" ]]; then
              list_of_pids_to_be_killed=$(fuser -c $MP)              
         else
               exportfs -u :${MP}
               list_of_pids_to_be_killed=$(fuser -m $MP)
         fi

         logger -i -p notice -t $myname "Following list of pids preventing $MP unmounting: $list_of_pids_to_be_killed" 
         sleep 10

         umount $MP
         doKillAlso=$?
         isMounted
         ((softUnmounts+=1))
      done

      # At this point, nice unmounting has not worked ... be more drastic

      while [ $mounted != 0 ];
      do
         if [[ "${UNAME}" == "AIX" ]]; then
            exportfs -u ${MP}
            list_of_pids_to_be_killed=$(fuser -c $MP)
        elif [[ "${UNAME}" == "SunOS" ]]; then
            list_of_pids_to_be_killed=$(fuser -c $MP)
        else
            exportfs -u :${MP}
            list_of_pids_to_be_killed=$(fuser -m $MP)
         fi

         if [[ $doKillAlso != 0 ]]; then 
            if [[ ! -z $list_of_pids_to_be_killed ]]; then
               logger -i -p notice -t $myname "Killing following list of pids: $list_of_pids_to_be_killed" 
               if [[ "${UNAME}" == "AIX" || "${UNAME}" == "SunOS" ]]; then
                  fuser -ck $MP 
               else
                  fuser -mk $MP 
               fi
            else
               # There are no PIDs holding this mount, yet umount fails 
               # Be even more drastic
               umount -f $MP
               isMounted
               if [ $mounted != 0 ]; then
                  logger -i -p error -t $myname "There are no PIDs preventing unmount, yet unmount fails!" 
               fi
            fi
         fi

         umount $MP
         doKillAlso=$?
         sleep 1
         isMounted
         if [ $mounted != 0 ]; then
            if [[ "${UNAME}" == "AIX"  || "${UNAME}" == "SunOS" ]]; then
               fuser -ck $MP 
            else
               fuser -mk $MP 
            fi
         fi
      done
   fi
fi

# Filesystem is unmounted or was not mounted
if [[ $UNAME == "AIX" && $concurrent != "Enhanced-Capable" ]]; then
   lspvLine=$(lspv -L | grep " $vgname ")
   vgactive=$(echo $lspvLine | awk '{print $4}')
   logger -i -p info -t $myname "Volume group $vgname state: $vgactive"
   while [ "X$vgactive" == "Xactive" ];
   do
      umount -f $MP
      openLV=$(lsvg -L ${vgname?} | grep "OPEN LVs" | awk '{print $3}')
      if [[ $openLV == 0 ]]; then
         varyoffvg ${vgname?}
      fi
      lspvLine=$(lspv -L | grep " $vgname ")
      vgactive=$(echo $lspvLine | awk '{print $4}')
      logger -i -p notice -t $myname "Volume group $vgname is still active : $vgactive"
   done
fi

if [[ $UNAME == "SunOS" ]]; then
   logger -i -p notice -t $myname "About to zpool export $poolName"

   rc=0
   zpool_already_imported=$(zpool list | awk '{print $1}' | grep -c "^${poolName?}$")
   logger -i -p info -t $myname "zpool ${poolName}: $zpool_already_imported"
   if [[ ${zpool_already_imported?} -ne 0 ]]; then
      zpool export ${poolName}
      rc=$?
      logger -i -p info -t $myname "zpool $poolName returns $rc"
   fi
fi

logger -i -p notice -t $myname "Returning $rc for $MP"
return $rc
